#include <iostream>
#include <vector>
using namespace std;

constexpr size_t K_par_defaut(3);

// ======================================================================
void affiche_synchro(vector<double> const& v,
                     vector<double> const& w)
{
  size_t offset(0);
  cout << "# signal ; moyenne mobile" << endl;
  if (w.size() < v.size()) {
    offset = v.size() - w.size();
    for (size_t i(0); i < offset; ++i) {
      cout << v[i] << endl;
    }
  }
  for (size_t i(offset); i < v.size(); ++i) {
    cout << v[i] << '\t' << w[i-offset] << endl;
  }
}

/* ======================================================================
 * Calcule le signal filtré par moyenne mobile sur fenêtre de taille T_c.
 */
vector<double> filtre_moyenne_mobile(vector<double> X,
                                     size_t K = K_par_defaut)
{
  // s'assurer qu'on ne fait pas de bêtises
  if ((K == 0)
      or (K > X.size())) // s'assurer qu'on a assez de points
    return vector<double>(); 

  vector<double> X_chapeau;

  // calcul initial
  double somme(0.0);
  for (size_t m(0); m < K; ++m) {
    somme += X[m];
  }
  X_chapeau.push_back(somme / K);

  // calcul de proche en proche utilisant la somme précédente
  for (size_t n(K); n < X.size(); ++n) {
    somme += X[n] - X[n-K];
    X_chapeau.push_back(somme / K);
  }
  
  return X_chapeau;
}

// ======================================================================
int main()
{
  vector<double> temperatures( { 15.1, 14.8, 13.7, 12.6, 13.8, 14.1,
                                 14.5, 14.8, 15.0, 15.1, 15.5         } );

  affiche_synchro(temperatures, filtre_moyenne_mobile(temperatures) );

  return 0;
}
